@@ -1,6 +1,6 @@
 package SPVM::Resource::Libpng;
 
-our $VERSION = "0.011";
+our $VERSION = "0.012";
 
 1;
 
@@ -40,7 +40,7 @@ L<libpng|https://github.com/glennrp/libpng>
 
 =head1 Original Product Version
 
-L<libpng v1.6.39|https://github.com/glennrp/libpng/releases/tag/v1.6.39>
+L<libpng v1.6.45|https://github.com/glennrp/libpng/releases/tag/v1.6.45>
 
 =head1 Language
 
@@ -114,22 +114,30 @@ C99
 
 =item * C<pngwutil.c>
 
+=item * C<arm/arm_init.c>
+
+=item * C<arm/filter_neon_intrinsics.c>
+
+=item * C<arm/filter_neon.S>
+
+=item * C<arm/palette_neon_intrinsics.c>
+
 =back
 
 =head1 How to Create Resource
 
 =head2 Donwload
 
-  mkdir -p original.tmp
-  git clone https://github.com/glennrp/libpng.git original.tmp/libpng
-  git -C original.tmp/libpng checkout tags/v1.6.39 -b branch_v1.6.39
-  git -C original.tmp/libpng branch
+  mkdir -p .tmp
+  git clone https://github.com/glennrp/libpng.git .tmp/libpng
+  git -C .tmp/libpng checkout tags/v1.6.45 -b branch_vv1.6.45
+  git -C .tmp/libpng branch
 
 =head1 Extracting Header Files
 
 The header files of C<libpng> is copied into the C<include> directory by the following command.
 
-  rsync -av --include='*.h' --exclude='*' original.tmp/libpng/ lib/SPVM/Resource/Libpng.native/include/
+  rsync -av --include='*.h' --exclude='*' .tmp/libpng/ lib/SPVM/Resource/Libpng.native/include/
   
   cp lib/SPVM/Resource/Libpng.native/src/scripts/pnglibconf.h.prebuilt lib/SPVM/Resource/Libpng.native/include/pnglibconf.h
 
@@ -137,7 +145,7 @@ The header files of C<libpng> is copied into the C<include> directory by the fol
 
 The source files of C<libpng> are copied into the C<src> directory by the following command.
 
-  rsync -av --exclude='*.h' original.tmp/libpng/ lib/SPVM/Resource/Libpng.native/src/
+  rsync -av --exclude='.git' .tmp/libpng/ lib/SPVM/Resource/Libpng.native/src/
 
 The used L<source files|/"Source Files"> are extracted by the following command.
 
